%% Class for Periodic Single Layer potential operator
%
% <latex>
% Define:
% $$S_D[f](x) = \int_{\partial D} G(x-y)f(y)ds(y)$$
% which is well defined in the whole space. In case that $x\notin \partial D$, the kernel matrix
% $$K_{ji} = G(x(t_j)-y(t_i)) \sigma_1(t_i),$$
% otherwise the diagonal term of the kernel matrix becomes:
% $$\frac{1}{2\pi} \sigma_1(t_i)\left(\log{\frac{\sigma_1(t_i)}{2}} - 1\right)$$
% </latex>
%

classdef SingleLayer_H_Periodic < ops.Operators
    % Class for single layer potential operator.
    
    methods
        function obj = SingleLayer_H_Periodic(k, D1, type1, step1, d, alpha)
            D2 = D1;
            type2 = type1;
            step2 = step1;
            
            obj = obj@ops.Operators(D1, type1, step1, D2, type2, step2);
            obj.Kmat = ops.SingleLayer_H_Periodic.make_kernel_matrix(k, D1.points, D1.sigma, D1.tvec, d, alpha);
        end
        
    end
    
    methods(Static)
        function K = make_kernel_matrix(k, D, sigma_D, tvec, d, alpha)
            % Kernel matrix of the single layer potential L^2(\p D) -> L^2(\p E).
            % Inputs:
            % D, sigma_D: boundary, and the integration elements.
            
            % Case of two identical boundaries. Although the
            % operator is stil well defined (no jumps), the diagonal
            % terms of the kernel matrix are computed in a special way.
            
            N = size(D,2);
            K = zeros(N) ;
                       
            for i=1:N
                Gi = -conj(ops.GSharp(conj(k), D(1,i)-D(1,:), D(2,i)-D(2,:), d, alpha));
                K(i,:) = sigma_D.*Gi;

                 if i == 1
                     K(i,i) = 1/2 * (K(i, N) +  K(i, 2));
                 elseif i == N
                     K(i,i) = 1/2 * (K(i, i-1) +  K(i, 1));
                 else
                    K(i,i) = 1/2 * (K(i, i-1) +  K(i, i+1));
                 end
            end
        end
        
        function [ val ] = eval(D, F, X)
            % Evaluate the single layer potential S_D[F](X)
            % INPUTS:
            % D: shape, a C2boundary object
            % F: function defined on D
            % X: points of evaluation of dimension 2X?. Must not contain any points of D.
            
            G = tools.Laplacian.Green2D(D.points, X);
            val = (reshape(F, 1, []) .* D.sigma) * G;
        end
    end
    
end

