% DiffractionGrating
%
% Overview:
%   We compute the scattered electric field for a 1d periodic dielectric grating.
%
% References:
%   Mathematical and Computational Methods in Photonics - Tutorial Notes
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Matias Ruiz, Sanghyeon Yu.

clc
clear all
close all

%% Material parameters
omega = 1;
epsM = 1;
epsC = 5;
muM = 1;
muC = 1;
kM = omega*sqrt(epsM * muM);
kC = omega*sqrt(epsC * muC);

%% Particle parameters
a = 0.4;
pointsN = 2^6;
D = shape.Ellipse(a, a, pointsN);
p = 1;
theta = pi/8;
    
%% Incident wave
alpha = kM*sin(theta);
beta = kM*cos(theta);
uIn = @(x1, x2) 3 * exp(1i * (alpha*x1 - beta*x2));

%% Boundary integral representation
I = eye(pointsN);
A11 = ops.SingleLayer_H_Periodic(kC, D, 'P0', 1, p, alpha);
A12 = ops.SingleLayer_H_Periodic(kM, D, 'P0', 1, p, alpha);
A21 = ops.Kstar_H_Periodic(kC, D, 'P0', 1, p, alpha);
A22 = ops.Kstar_H_Periodic(kM, D, 'P0', 1, p, alpha);

A = [ A11.Kmat, -1 * A12.Kmat ; 1/muC * (-1/2*I + A21.Kmat), -1/muM * (1/2*I + A22.Kmat) ];

b1 = zeros(pointsN, 1);
b2 = zeros(pointsN, 1);
for j=1:pointsN
    b1(j) = uIn(D.points(1, j), D.points(2, j));
    b2(j) = dot(D.normal(:, j),...
        [ 1i * alpha * uIn(D.points(1, j), D.points(2, j));...
        -1i * beta * uIn(D.points(1, j), D.points(2, j));  ]);
end

vecB = [ b1; 1/muM*b2 ];

sol = A\vecB; 
phi = sol(1:pointsN);
psi = sol(pointsN + 1:2*pointsN);

%% Calculate field over a grid
gridN = 96;
gridMinX1 = -3*pi;
gridMaxX1 = 3*pi;
gridMinX2 = -3*pi;
gridMaxX2 = 3*pi;
g1 = linspace(gridMinX1, gridMaxX1, gridN);
g2 = linspace(gridMinX2, gridMaxX2, gridN);
[ g1, g2 ] = meshgrid(g1, g2);
gridPoints = [g1(:) g2(:)]';

gridPointsN = length(gridPoints);

uIncident = zeros(gridPointsN, 1);
uInternal = zeros(gridPointsN, 1);
uScattered = zeros(gridPointsN, 1);
uTotal = zeros(gridPointsN, 1);
for j = 1:gridPointsN
    gridPoint = gridPoints(:, j);
    
    % Determine whether we are inside or outside the periodic array of
    % particles
    flcl = round(gridPoint(1));
    test = abs(gridPoint(1) - flcl);
    if abs(gridPoint(2)) <= p/2 && test^2 + gridPoint(2)^2  <= a^2  
        S = -conj(ops.GSharp(conj(kC), gridPoints(1, j)-D.points(1,:), gridPoints(2,j)-D.points(2,:), p, alpha)) .* D.sigma;
        S = S * phi;
        uInternal(j) = S;
        uIncident(j) = uIn(gridPoints(1, j), gridPoints(2, j));
        uTotal(j) = uInternal(j);
    else
        S = -conj(ops.GSharp(conj(kM), gridPoints(1, j)-D.points(1,:), gridPoints(2,j)-D.points(2,:), p, alpha)) .* D.sigma;
        S = S * psi;
        uIncident(j) = uIn(gridPoints(1, j), gridPoints(2, j));
        uScattered(j) = S;
        uTotal(j) = uScattered(j) + uIncident(j);
    end
end

uIncident = reshape(uIncident, [gridN gridN]);
uInternal = reshape(uInternal, [gridN gridN]);
uScattered = reshape(uScattered, [gridN gridN]);
uTotal = reshape(uTotal, [gridN gridN]);

%% Plot field
hFig = figure(21);
set(hFig, 'Position', [100 100 1200 900]);
subplot(2, 2, 1)
surf(g1, g2, real(uIncident), 'edgecolor', 'none'); xlabel('x1'); ylabel('x2'); %title('u^{incident}')
axis([gridMinX1, gridMaxX1, gridMinX2, gridMaxX2, min(real(uIncident(:))), max(real(uIncident(:))) ]); view([0, 0, 90])
subplot(2, 2, 2)
surf(g1, g2, real(uScattered), 'edgecolor', 'none'); xlabel('x1'); ylabel('x2'); %title('u^{scattered}')
axis([gridMinX1, gridMaxX1, gridMinX2, gridMaxX2, min(real(uScattered(:))), max(real(uScattered(:))) ]); view([0, 0, 90])
subplot(2, 2, 3)
surf(g1, g2, real(uInternal), 'edgecolor', 'none'); xlabel('x1'); ylabel('x2'); %title('u^{internal}')
axis([gridMinX1, gridMaxX1, gridMinX2, gridMaxX2, min(real(uInternal(:))), max(real(uInternal(:))) ]); view([0, 0, 90])
subplot(2, 2, 4)
surf(g1, g2, real(uTotal), 'edgecolor', 'none'); xlabel('x1'); ylabel('x2'); %title('u')
axis([gridMinX1, gridMaxX1, gridMinX2, gridMaxX2, min(real(uTotal(:))), max(real(uTotal(:))) ]); view([0, 0, 90])
rotate3d on
